from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer


class IntidSetupFixture(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # pylint: disable=W0613
        import plone.app.intid

        self.loadZCML(package=plone.app.intid)


SETUP_TESTING = IntegrationTesting(
    bases=(IntidSetupFixture(),), name="IntidSetupFixture:Setup"
)



from plone.app.intid.testing import SETUP_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.fti import DexterityFTI
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unittest


class TestSetup(unittest.TestCase):
    layer = SETUP_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        # XXX below code is only needed if there's no Folder FTI already setup.
        typetool = getToolByName(self.portal, "portal_types")
        if "Folder" not in typetool.objectIds():
            # XXX Check if this is needed for Plone 5.0! In 4.3 the FTI is
            # already setup
            fti = DexterityFTI("Folder")
            typetool._setObject("Folder", fti)

    def tearDown(self):
        setRoles(self.portal, TEST_USER_ID, ["Member"])

    def test_already_installed(self):
        """plone.app.intid is a dependency of plone.app.linkintegrity
        which is a dependency of CMFPlone, so it is always installed.
        This tests if this is true.
        """
        # we create a folder
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        folder_id = self.portal.invokeFactory("Folder", "folder")
        folder = self.portal[folder_id]
        intids = getUtility(IIntIds)
        self.assertIsNotNone(intids.getId(folder))

    @unittest.skip("p.a.intid is always installed")
    def test_install(self):
        """When p.app.intid is installed it registers some utility
        from zope.intid and five.intid and search in portal_catalog
        all contents in order to register them in these utilities.

        This test checks that all pre existing contents
        are registered correctly
        """
        from plone.app.intid.setuphandlers import add_intids
        from plone.app.testing import applyProfile

        # we create a folder before the installation of plone.app.intid
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        folder_id = self.portal.invokeFactory("Folder", "folder")
        folder = self.portal[folder_id]

        # now we install manually the intid utilities
        add_intids(self.portal)
        intids = getUtility(IIntIds)

        # this folder is not referenced by intid utility
        self.assertRaises(KeyError, intids.getId, folder)

        # when we install p.app.intid our folder is referencend by intid
        applyProfile(self.portal, "plone.app.intid:default")
        self.assertIsNotNone(intids.getId(folder))



from five.intid.intid import IntIds
from five.intid.site import addUtility
from Products.CMFCore.interfaces import IContentish
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import logging


logger = logging.getLogger(__name__)


def register_all_content_for_intids(portal):
    """Registers all existing content with the intid utility.
    This will not be fast."""
    cat = getToolByName(portal, "portal_catalog", None)
    if cat is None:
        return
    intids = getUtility(IIntIds)
    # Take advantage of paths stored in keyreferences in five.intid to optimize
    # registration
    registered_paths = {ref.path for ref in intids.ids if hasattr(ref, "path")}
    # Count how many objects we register
    registered = 0
    existing = 0
    query = {"object_provides": IContentish.__identifier__}
    for brain in cat(query):
        if brain.getPath() in registered_paths:
            existing += 1
            continue
        try:
            obj = brain.getObject()
            intids.register(obj)
            registered += 1
        except (AttributeError, KeyError, TypeError):
            # "TypeError" happens on a "could not adapt" - this may happen
            # for some contenttypes and must not stop this from working.
            logger.exception(brain.getURL())
    return registered, existing


def add_intids(context):
    addUtility(context, IIntIds, IntIds, ofs_name="intids", findroot=False)


def installIntIds(context):
    if context.readDataFile("install_intids.txt") is None:
        return
    portal = context.getSite()
    add_intids(portal)
    return "Added intid utility."


def registerContent(context):
    if context.readDataFile("intid_register_content.txt") is None:
        return
    portal = context.getSite()
    registered, existing = register_all_content_for_intids(portal)
    return (
        "Assigned intids to {} content objects, {} objects "
        "already had intids.".format(registered, existing)
    )



from setuptools import find_packages
from setuptools import setup

import os


version = "2.0.0"

setup(
    name="plone.app.intid",
    version=version,
    description="Installation and migration support for five.intid within " "Plone/CMF",
    long_description="{}\n{}".format(
        open("README.rst").read(),
        open(os.path.join("CHANGES.rst")).read(),
    ),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    keywords="plone zope five intid",
    author="Alec Mitchell",
    author_email="apm13@columbia.edu",
    url="https://github.com/plone/plone.app.intid",
    license="GPL",
    packages=find_packages(),
    namespace_packages=["plone", "plone.app"],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    install_requires=[
        "setuptools",
        "zope.intid",
        "zope.lifecycleevent",
        "five.intid>=1.0",
        "Products.CMFCore",
        "Products.GenericSetup",
    ],
    extras_require={
        "test": [
            "plone.app.testing",
            "plone.dexterity",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
